@@ -21,7 +21,7 @@ import keyword
 from typing import Set, Sequence, List, Callable, Optional, Union, Iterable
 
 from types import ModuleType
-from typing import Callable, Iterable, List, Optional, Sequence, Set, Union
+from typing import Callable, Iterable, List, Optional, Sequence, Set, Union, Dict
 
 import pkg_resources
 import regex as re
@@ -52,6 +52,13 @@ class OpenSCADObject:
         self.is_hole = False
         self.has_hole_children = False
         self.is_part_root = False
+        self.traits: Dict[str, Dict[str, float]] = {}
+
+    def add_trait(self, trait_name:str, trait_data:Dict[str, float]):
+        self.traits[trait_name] = trait_data
+
+    def get_trait(self, trait_name:str) -> Optional[Dict[str, float]]:
+        return self.traits.get(trait_name)
 
     def set_hole(self, is_hole: bool = True) -> "OpenSCADObject":
         self.is_hole = is_hole
@@ -398,9 +405,13 @@ def _find_include_strings(obj: Union[IncludedOpenSCADObject, OpenSCADObject]) ->
         include_strings.add(obj.include_string)
     for child in obj.children:
         include_strings.update(_find_include_strings(child))
+    # We also accept IncludedOpenSCADObject instances as parameters to functions, 
+    # so search in obj.params as well
+    for param in obj.params.values():
+        if isinstance(param, OpenSCADObject):
+            include_strings.update(_find_include_strings(param))
     return include_strings
 
-
 def scad_render(scad_object: OpenSCADObject, file_header: str = '') -> str:
     # Make this object the root of the tree
     root = scad_object
@@ -418,7 +429,6 @@ def scad_render(scad_object: OpenSCADObject, file_header: str = '') -> str:
 
     return file_header + includes + scad_body
 
-
 def scad_render_animated(func_to_animate: AnimFunc, 
                          steps: int =20, 
                          back_and_forth: bool=True, 
@@ -481,7 +491,6 @@ def scad_render_animated(func_to_animate: AnimFunc,
                            f"}}\n"
     return rendered_string
 
-
 def scad_render_animated_file(func_to_animate:AnimFunc, 
                               steps: int=20, 
                               back_and_forth: bool=True, 
@@ -553,7 +562,6 @@ def _write_code_to_file(rendered_string: str,
     out_path.write_text(rendered_string)
     return out_path.absolute().as_posix()
 
-
 def _get_version() -> str:
     """
     Returns SolidPython version
@@ -607,7 +615,6 @@ def extract_callable_signatures(scad_file_path: PathStr) -> List[dict]:
     scad_code_str = Path(scad_file_path).read_text()
     return parse_scad_callables(scad_code_str)
 
-
 def parse_scad_callables(scad_code_str: str) -> List[dict]:
     callables = []
 
@@ -646,16 +653,15 @@ def parse_scad_callables(scad_code_str: str) -> List[dict]:
             arg_matches = re.finditer(args_re, all_args)
             for am in arg_matches:
                 arg_name = am.group('arg_name')
-                if am.group('default_val'):
-                    kwargs.append(arg_name)
-                else:
-                    args.append(arg_name)
+                # NOTE: OpenSCAD's arguments to all functions are effectively
+                # optional, in contrast to Python in which all args without
+                # default values are required.
+                kwargs.append(arg_name)
 
         callables.append({'name': callable_name, 'args': args, 'kwargs': kwargs})
 
     return callables
 
-
 def calling_module(stack_depth: int = 2) -> ModuleType:
     """
     Returns the module *2* back in the frame stack.  That means:
@@ -678,7 +684,6 @@ def calling_module(stack_depth: int = 2) -> ModuleType:
         import __main__ as calling_mod  # type: ignore
     return calling_mod
 
-
 def new_openscad_class_str(class_name: str,
                            args: Sequence[str] = None,
                            kwargs: Sequence[str] = None,
@@ -731,7 +736,6 @@ def new_openscad_class_str(class_name: str,
 
     return result
 
-
 def _subbed_keyword(keyword: str) -> str:
     """
     Append an underscore to any python reserved word.
@@ -744,7 +748,6 @@ def _subbed_keyword(keyword: str) -> str:
               f"can be accessed with `{new_key}` in SolidPython\n")
     return new_key
 
-
 def _unsubbed_keyword(subbed_keyword: str) -> str:
     """
     Remove trailing underscore for already-subbed python reserved words.
@@ -753,11 +756,9 @@ def _unsubbed_keyword(subbed_keyword: str) -> str:
     shortened = subbed_keyword[:-1]
     return shortened if shortened in PYTHON_ONLY_RESERVED_WORDS else subbed_keyword
 
-
 # now that we have the base class defined, we can do a circular import
 from . import objects
 
-
 def py2openscad(o: Union[bool, float, str, Iterable]) -> str:
     if type(o) == bool:
         return str(o).lower()
@@ -768,6 +769,8 @@ def py2openscad(o: Union[bool, float, str, Iterable]) -> str:
     if type(o).__name__ == "ndarray":
         import numpy  # type: ignore
         return numpy.array2string(o, separator=",", threshold=1000000000)
+    if isinstance(o, IncludedOpenSCADObject):
+        return o._render()[1:-1]
     if hasattr(o, "__iter__"):
         s = "["
         first = True
@@ -780,6 +783,5 @@ def py2openscad(o: Union[bool, float, str, Iterable]) -> str:
         return s
     return str(o)
 
-
 def indent(s: str) -> str:
     return s.replace("\n", "\n\t")
